# ROC Curves for nbest AUC (leave one out) SVM
# 
#
# Author: Julien Gagneur
###############################################################################

require(ROCR)
require(e1071)

## ROC Curves for nbest AUC (leave one out) SVM
## out of those provided in parameters
## returns all auc values
bestAUC = function(parameters, feature, e2i, tiss, kernel="radial", max.prop.SV = 0.9, nbest = 10, verbose=TRUE, plot=TRUE, plot.folder ){
    ## first check for which combination of parameters the number of SVs is small enough while trained on the whole dataset
    sv = apply(
            parameters,
            1,
            function(param){
                f = fitSVM( feature, e2i[,tiss], kernel= kernel, cost = param[1], gamma =param[2])
                sum(f$nSV)
            }
    )
    
    ind = which(sv<max.prop.SV*nrow(feature))
    
    ## then do the leave-one-out validation for the interesting cases
    score = t(sapply(
                    1:nrow(feature),
                    function(i){
                        if(verbose) cat(".")
                        subset = setdiff( 1:nrow(feature), i )
                        fits = lapply(
                                ind,							
                                function(j)								
                                    fitSVM( feature[subset,], e2i[subset,tiss], kernel= kernel, cost = parameters$cost[j], gamma =parameters$gamma[j])
                        )
                        score = unlist(lapply( fits, function(f) predSVM(f,feature[i,, drop=FALSE]) ) )					
                        score
                    }
            ))
    
    colnames(score) = paste( "C=",sprintf("%1.1e", parameters$cost[ind]), ", g=", sprintf("%1.1e", parameters$gamma[ind]), sep="")
    
    ######################
    ## AUC
    ######################
    cost = sort( unique(parameters$cost) )
    gamma = sort( unique(parameters$gamma) )
    
    nr = length(cost)
    nc = length(gamma)
    
    auc = unlist(lapply(
                    1:ncol(score),
                    function(i){
                        pred <- prediction( score[,i], ifelse(e2i[[tiss]] ==1,1,-1) )
                        performance(pred, "auc")@y.values[[1]]
                    }
            ))
    auc.val = rep(as.numeric(NA),nrow(parameters) )
    auc.val[ind] = auc
    auc.mat = matrix(auc.val, nr = nr, nc=nc)
    
    ## plot AUC map, ROC plots
    if(plot){
        
        pdf( file.path( plot.folder, paste("auc_roc_",tiss,".pdf",sep="") ), width=8, height=8 )
        ## plot of auc
        filled.contour(
                1:nr,
                1:nc,
                auc.mat,
                plot.axes = {
                    axis(1, 1:nr, sprintf("%1.2e", cost), cex.axis=0.7 )
                    axis(2, 1:nc, sprintf("%1.2e", gamma), cex.axis=0.7)
                },
                xlab="cost",
                ylab="gamma",
                color = terrain.colors,
                main = paste("AUC", tiss)
        )
        
        ######################
        ## ROC plot for the nbest best
        ######################
        
        auc.min = sort(auc, decreasing=TRUE)[nbest]
        roc.score = score[,auc>=auc.min,drop=FALSE]
        
        cols = rainbow(ncol(roc.score))
        pred <- prediction( roc.score[,1], ifelse(e2i[[tiss]] ==1,1,-1) )
        perf <- performance(pred, "tpr", "fpr")
        plot(perf, col = cols[1], main =tiss)
        for(i in 2:ncol(roc.score)){
            pred <- prediction( roc.score[,i], ifelse(e2i[[tiss]] ==1,1,-1) )	
            perf <- performance(pred, "tpr", "fpr")
            plot(perf, col = cols[i], add=TRUE)	
        }
        abline(a=0,b=1)
        
        legend(
                "bottomright",
                col=cols,
                lty = 1,
                legend = colnames(roc.score)
        )
        
        dev.off()
        
    }
    
    return( data.frame( parameters, auc.val ) ) 
}
