# 
# sensitivity and specificity estimation for the chosen SVM parameters
#
#
#
# Author: julien.gagneur
###############################################################################

library(e1071)
library(ROCR)

source("R/svm_funcs.R")

#####################################################
# settings
#####################################################

folder = "data"
plot.folder = file.path(folder, "plot_sens_spec")
if(!file.exists(plot.folder)) dir.create(plot.folder)

#####################################################
# help functions
#####################################################

# Normal reference rule for kernel estimate of CDF
# from http://hivdb.stanford.edu/PhenotypeComparison/statScripts/ROC_lib/ROC.R

h.lloyd2 <- function(x)
{
    n <- length(x)
    s <- IQR(x) / 1.349
    return(1.583*s/n^(1/3))
}

## return the CDF function by Gaussian kernel density estimation 
makeCDF <- function( x, h=h.lloyd2(x) )
{
    return(
            function(y){
                n = length(x)
                z = sapply(x, function(xi) pnorm( (y-xi)/h ) )
                (1/n)*sum( z )
            }
    )
}

#####################################################
# load data
#####################################################

## read training set
tab8 = read.delim( file.path(folder, "supplementary_table_8_training_set.txt"), comment.char="#" )
elem2insitu = tab8[, 1:11]
x = as.matrix( tab8[, 12:ncol(tab8) ] )

## svm parameters
load( file.path(folder, "svmParams.rda") )

## CRM Atlas, ordered by CRM_id
## bm: binding matrix
tab5 = read.delim( file.path(folder, "supplementary_table_5_CRM_atlas.txt"), comment.char="#" )
tab5 = tab5[order(tab5[,1]), ]
bm = tab5[, 5:ncol(tab5)]

scores = sens = spec = list()

# coefficient for bandwidth (relative to Lloyd's bw)
k = 0.5

for(i in 1:nrow(svmParams)){
    tissue = svmParams$tissue[i]
    
    cat("tissue", tissue, "\n")
    
    ## train SVM on full dataset, apply on complete CRM Atlas
    param = svmParams[svmParams$tissue==tissue, ]
    fit = fitSVM( x, elem2insitu[[tissue]], kernel= param$kernel, cost = param$cost, gamma =param$gamma) 
    score = predSVM( fit, bm)
    sl = range(score)
        
    ## leave-one-out validation
    print("leave-one-out validation")
    looscore = sapply(
            1:nrow(x),
            function(j){
                subset = setdiff( 1:nrow(x), j )
                f =  fitSVM( x[subset,], elem2insitu[subset,tissue], kernel= svmParams$kernel[i], cost = svmParams$cost[i], gamma = svmParams$gamma[i])
                predSVM(f,x[j,, drop=FALSE])
            }
    )
    
    #####################################################
    # sensitivity and specificity function estimation
    #####################################################
    
    print("CDF by kernel")
    ### get CDF of ranks for negative data
    cdf =  list()
    cdf[[0+1]] = function(y){
        x = looscore[ elem2insitu[[tissue]]==0 ]
        h = k*h.lloyd2(x) 
        n = length(x)
        z = sapply(x, function(xi) pnorm( (y-xi)/h ) )
        (1/n)*sum( z )
    }
    cdf[[1+1]] = function(y){
        x = looscore[ elem2insitu[[tissue]]==1 ]
        h = k*h.lloyd2(x) 
        n = length(x)
        z = sapply(x, function(xi) pnorm( (y-xi)/h ) )
        (1/n)*sum( z )
    }
    
    
    
    ## plot CDF fit
    png( file.path(plot.folder, paste(tissue, "_looECDF.png", sep="")) )
    par( mfrow=c(2,1))
    for(i in 0:1){
        plot( ecdf( looscore[elem2insitu[[tissue]] ==i ]), xlim=sl, main=paste( tissue,"leave-one-out ECDF vs fitted (red)\n", "class",i), xlab ="SVM score" )
        lines(seq(sl[1],sl[2],0.1), sapply( seq(sl[1],sl[2],0.1), cdf[[ i + 1]] ), col="red")
    }
    dev.off()
    
    ## plot ROC with smooth fit
    png( file.path(plot.folder, paste(tissue, "_ROC.png", sep="") ) )		
    
    pred <- prediction( looscore, ifelse(elem2insitu[[tissue]] ==1,1,-1) )
    perf <- performance(pred, "tpr", "fpr")
    plot( perf, colorize=TRUE, lwd=2, main = paste(tissue, "\nleave-one-out performance" ) )
    abline(a=0,b=1)
    lines( 1-sapply( seq(sl[1],sl[2],0.1), cdf[[ 0 + 1]] ), 1-sapply( seq(sl[1],sl[2],0.1), cdf[[ 1 + 1]] ), col="darkgrey")
    
    dev.off()
    
    ## compute sens/spec for all CRMs score on final output
    print("leave-one-out pvals for all CRMs score on final output")
    spec[[tissue]]  = sapply(score, cdf[[1]] )
    sens[[tissue]] = 1-sapply(score, cdf[[2]] )
    scores[[tissue]] = score
    
}# for tissue...

pvals = lapply( spec, function(x) 1-x)

## make result table
res = tab5
for(i in 1:nrow(svmParams)){
    tissue = svmParams$tissue[i]
    res[[ paste(tissue, "_score",sep="") ]] = scores[[tissue]]
    res[[ paste(tissue, "_spec",sep="") ]] = spec[[tissue]]
    res[[ paste(tissue, "_sens",sep="") ]] = sens[[tissue]]
}

save(res, file=file.path(folder,paste("res", ".rda", sep="")) )
